package turgen.plugin.csturbo;

import turgen.plugin.*;
import turgen.generator.SignalGenerator;
import turgen.config.Configuration;
import turgen.*;
import java.io.*;
import javax.swing.*;
import turgen.config.BooleanConfigEntry;
import turgen.config.ConfigClass;
import turgen.config.IntegerConfigEntry;
import turgen.dtb.*;
import turgen.generator.PulseCrate;
import turgen.generator.PulseCreator;
import turgen.playlist.AbstractPlayListItem;
import turgen.tapeimage.DataChunk;
import turgen.tapeimage.TapeImageChunk;
import turgen.tapeimage.TurboChunk;

public class SuperTurboPlugin implements Plugin {

    
    private String lastError;
    private CsTurboPluginsPanel panel;
    private InOutProvider ioProvider;
    private int[] casChunkName = {116, 114, 115, 116};

    private ConfigClass cfcSuperTurbo;
    private IntegerConfigEntry cfgPilotLenA;
    private IntegerConfigEntry cfgPilotLenB;
    private BooleanConfigEntry cfgProlongatePilotTone;
    private IntegerConfigEntry cfgPilotLenBlockLoading;
    private BooleanConfigEntry cfgProlongateBlockLoadingPilotTone;
    private BooleanConfigEntry cfgPrependLoader;

    /** Nova instance SuperTurboPlugin */
    public SuperTurboPlugin() {
        
    }

    /** Jake rychlosti jsou podporovany*/
    

    /** Identifikacni jmeno modulu*/
    public String getName() {
        return "Super Turbo";

    }

    /*************************************************************/
    /*Tvorba instrukci pro generator*/
    public int[] getInstructions(AbstractPlayListItem _pl) {

        /*Get CsTurboPlaylistItem*/
        CsTurboPlaylistItem pl = (CsTurboPlaylistItem) _pl;

        
        RandomAccessFile raf = null;
        

        try {


            /*Pripraven vystupni proud pro instrukce*/
            InstructionStream is = new InstructionStream();
            
            /*If loader is to be prepended, then we prepend it*/
            if (cfgPrependLoader.getAsBoolean()==true) {
                is.add(LoaderBinaryData.universalTurboLoader);
            }

            /*Nastaveni, pro vsechny mozne konverze je nastaveni stejne,
             *bloky nebo soubory jsou vzdy v Super Turbu o dane rychlosti
             */
            Utils.addSetupInstruction(is, pl.pulses, casChunkName,SignalGenerator.FLAG_POLARITY_10,SignalGenerator.FLAG_ORDER_HL);

            /*Nyni se vybere konverze podle typu v polozce playlistu*/
            switch (pl.conversionType) {

                /*Zde se prevadi cely soubor*/
                case CsTurboPlaylistItem.CV_BINTURBO:
                case CsTurboPlaylistItem.CV_BASIC:
                case CsTurboPlaylistItem.CV_DATA: {
                    convertFullFile(is, pl);
                    break;
                }
                /*Monoliticky binarni soubor*/
                case CsTurboPlaylistItem.CV_MONOLITHIC: {
                    convertMonolithicBinary(is, pl);
                    break;
                }
                /*Specialni retezove zavadeni*/
                case CsTurboPlaylistItem.CV_SEGMENTED_CHAIN: {
                    convertChainLoading(is, pl);
                    break;
                }
                /*Specialni blokove zavadeni*/
                case CsTurboPlaylistItem.CV_SEGMENTED_BLOCK: {
                    convertBlockLoading(is, pl);
                    break;
                }
            }

            is.add(SignalGenerator.INSTR_END);

            
            /*Instrukce pro generatory*/
            return is.getInstructions();

        } catch (Exception e) {
            /*Uklidit*/
            try {
                raf.close();
            } catch (Exception xe) {
            }
            e.printStackTrace();
            if (e.getMessage() == null) {
                lastError = "Unable to produce instructions: " + e.toString();
            } else {
                lastError = e.toString();
            }
            return null;

        }
    }

    public String getLastError() {
        return lastError;
    }

    public AbstractPlayListItem createItem() {
        CsTurboPlaylistItem dpr = panel.getRecord();
        dpr.pluginClass = Turgen.pluginManager.getPluginClassName(this);
        return dpr;
    }

    public void resetPanel() {
        panel.clear();
    }

    public String validateInput() {
        return panel.verify();
    }

    public JPanel getItemPanel() {
        return panel;
    }

    public void init() {
        panel = new CsTurboPluginsPanel(true);

        /*Initialize configuration class*/
        cfcSuperTurbo = new ConfigClass("SuperTurbo","Super Turbo");

        /*Initialize configuration entries*/
        cfgPilotLenA = new IntegerConfigEntry("pilotLenA","Header block pilot tone length [pulse]", 3072, 256, 8192);
        cfgPilotLenB = new IntegerConfigEntry("pilotLenB","Data block pilot tone length [pulse]", 3072, 256, 8192);
        cfgProlongatePilotTone = new BooleanConfigEntry("prolongatePilotTone","Prolongate pilot tone", true);
        cfgPilotLenBlockLoading = new IntegerConfigEntry("pilotLenBlockLoading","Pilot tone length for BlockLoading [pulse]", 2048, 256, 8192);
        cfgProlongateBlockLoadingPilotTone = new BooleanConfigEntry("prolongateBlockLoadingPilotTone","Prolongate pilot tone for BlockLoading", true);
        cfgPrependLoader = new BooleanConfigEntry("prependLoader","Prepend Universal Turbo loader",false);

        /*Add entries to the class*/
        cfcSuperTurbo.addEntry(cfgPilotLenA);
        cfcSuperTurbo.addEntry(cfgPilotLenB);
        cfcSuperTurbo.addEntry(cfgPilotLenBlockLoading);
        cfcSuperTurbo.addEntry(cfgProlongatePilotTone);
        cfcSuperTurbo.addEntry(cfgProlongateBlockLoadingPilotTone);
        cfcSuperTurbo.addEntry(cfgPrependLoader);

        /*Register configuration class*/
        Configuration.registerClass(cfcSuperTurbo);
    }

    public void fillPanel(AbstractPlayListItem plr) {
        CsTurboPlaylistItem dpr = (CsTurboPlaylistItem) plr;
        panel.fill(dpr);
    }

    public void setInOutProvider(InOutProvider prov) {
        ioProvider = prov;
        panel.ioProvider = prov;
    }

    private void convertFullFile(InstructionStream is, CsTurboPlaylistItem cpi) throws Exception {

        int hdrLoad = Integer.parseInt(cpi.load);
        int hdrRun = Integer.parseInt(cpi.run);
        int hdrLength = Integer.parseInt(cpi.length);
        int hdrType = Integer.parseInt(cpi.type);

        int plenA = cfgPilotLenA.getAsInt();
        int plenB = cfgPilotLenA.getAsInt();

        if (cfgProlongatePilotTone.getAsBoolean() == true) {
            plenA = PulseCreator.prolongatePilotTone(plenA, 32,cpi.pulses.pilotTonePulseWidth);
            plenB = PulseCreator.prolongatePilotTone(plenB, 32,cpi.pulses.pilotTonePulseWidth);
        }

        /*HLAVICKA*/
        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(plenA);
        is.add(SignalGenerator.INSTR_SYNC);

        is.add(SignalGenerator.INSTR_DATA);
        is.add(29);

        int[] hdr = Utils.createSTHeader(cpi.fname, hdrType, hdrLoad, hdrLength, hdrRun);
        is.add(hdr);
        

        /*Safety pulse + safety pause*/
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);

        /*DATA*/
        int[] fileData = readFileData(cpi.inFile, 0, Integer.MAX_VALUE);

        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(plenB);
        is.add(SignalGenerator.INSTR_SYNC);

        is.add(SignalGenerator.INSTR_DATA);
        is.add(fileData.length + 2);

        int chsum = 237;
        is.add(chsum);
        for (int i = 0; i < fileData.length; i++) {
            chsum ^= fileData[i];
        }
        is.add(fileData);
        is.add(chsum);

        /*Safety pulse + safety pause*/
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);

    }

    private void convertMonolithicBinary(InstructionStream is, CsTurboPlaylistItem cpi) throws Exception {

        int hdrLoad = Integer.parseInt(cpi.load);
        int hdrRun = Integer.parseInt(cpi.run);
        int hdrLength = Integer.parseInt(cpi.length);
        int hdrType = Integer.parseInt(cpi.type);

        int plenA = cfgPilotLenA.getAsInt();
        int plenB = cfgPilotLenA.getAsInt();

        if (cfgProlongatePilotTone.getAsBoolean() == true) {
            plenA = PulseCreator.prolongatePilotTone(plenA, 32,cpi.pulses.pilotTonePulseWidth);
            plenB = PulseCreator.prolongatePilotTone(plenB, 32,cpi.pulses.pilotTonePulseWidth);
        }

        /*HLAVICKA*/
        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(plenA);
        is.add(SignalGenerator.INSTR_SYNC);

        is.add(SignalGenerator.INSTR_DATA);
        is.add(29);

        int[] hdr = Utils.createSTHeader(cpi.fname, hdrType, hdrLoad, hdrLength, hdrRun);
        is.add(hdr);
        

        /*Safety pulse + safety pause*/
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);

        /*DATA*/
        DOS2Binary dtb = new DOS2Binary(cpi.inFile);
        dtb.analyzeFromFile();

        int[] fileData = dtb.getFirstDataSegment().data;

        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(plenB);
        is.add(SignalGenerator.INSTR_SYNC);

        is.add(SignalGenerator.INSTR_DATA);
        is.add(fileData.length + 2);

        int chsum = 237;
        is.add(chsum);
        for (int i = 0; i < fileData.length; i++) {
            chsum ^= fileData[i];
        }
        is.add(fileData);
        is.add(chsum);
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);
    }

    private void convertChainLoading(InstructionStream is, CsTurboPlaylistItem cpi) throws Exception {


        int currentPause = 0;
        int plen = cpi.silenceList.length;


        /*Nejprve se vytvori instance DOS2Binary*/
        DOS2Binary ae = new DOS2Binary(cpi.inFile);
        ae.analyzeFromFile();
        ae.prepareForDumbLoader();

        /*Mirne navyseni poctu zavadecich pulzu*/

        int currentPilotLenA = cfgPilotLenA.getAsInt();
        int currentPilotLenB = cfgPilotLenB.getAsInt();

        if (cfgProlongatePilotTone.getAsBoolean() == true) {
            currentPilotLenA = PulseCreator.prolongatePilotTone(currentPilotLenA, 32,cpi.pulses.pilotTonePulseWidth);
            currentPilotLenB = PulseCreator.prolongatePilotTone(currentPilotLenB, 32,cpi.pulses.pilotTonePulseWidth);
        }

        /*CHAINLOADER*/
        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(currentPilotLenA);
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_DATA);
        is.add(29);

        /*Adresy*/
        int ls = 0;
        int ll = 0;
        int lr = 0;
        lr = superChain_run;
        ll = superChain_data.length;
        ls = superChain_load;

        int[] header = Utils.createSTHeader(cpi.fname, 3, ls, ll, lr);

        /*Zapsat do proudu*/
        is.add(header);
       

        /*Safety pulse + safety pause*/
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);

        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(currentPilotLenB);
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_DATA);
        is.add(ll + 2);

        /*Data zavadece*/
        int p = 237;
        is.add(237);

        for (int k = 0; k < ll; k++) {
            p = p ^ superChain_data[k];
        }
        
        is.add(superChain_data);
        /*Kontroni soucet*/
        is.add(p);
        is.add(SignalGenerator.INSTR_NARROW);
        /*Pauza*/
        is.add(SignalGenerator.INSTR_LONGSILENCE);
        is.add(3);

        /*Instrukce pro vygenerovani zavadece jsou jiz hotove*/
        int currentSegment = 0;
        int realCounter = 0;

        boolean initTag = false;

        while (currentSegment < ae.allSegments.size()) {

            /*Vezmeme segment*/
            Segment sct = (Segment) ae.allSegments.get(currentSegment);
            initTag = false;

            /*Zavadeci tony*/
            is.add(SignalGenerator.INSTR_PILOTTONE);
            is.add(currentPilotLenA);
            is.add(SignalGenerator.INSTR_NARROW);
            is.add(SignalGenerator.INSTR_DATA);
            is.add(29);


            /*Do hlavicky radeji cislo segmentu*/
            String s1 = Integer.toString(currentSegment);
            String s2 = Integer.toString(ae.allSegments.size() - 1);
            s1 = " " + s1 + "/" + s2;
            if (s1.length() > 8) {
                s1 = s1.substring(0, 8);
            }
            header = Utils.createSTHeader(s1, 3, 0, 0, 0);

            int progress = 1;

            /*Podle typu segmentu*/
            switch (sct.type) {
                /*Datovy segment*/
                case (Segment.DATA_SEGMENT): {

                    int mark = 68;
                    lr = sct.start;
                    /*Neni-li to posledni segment*/
                    if (currentSegment < ae.allSegments.size() - 1) {

                        /*Podivat se jestli nenasleduje INIT nebo RUN*/
                        Segment js = (Segment) ae.allSegments.get(currentSegment + 1);
                        if (js.type == Segment.RUN_SEGMENT) {
                            lr = js.jump;
                            mark = 88;
                            progress = 2;
                            if (currentSegment == ae.allSegments.size() - 2) {
                                mark += 128;
                            }
                        }
                        if (js.type == Segment.INIT_SEGMENT) {
                            lr = js.jump;
                            mark = 89;
                            progress = 2;
                            initTag = true;
                            if (currentSegment == ae.allSegments.size() - 2) {
                                mark += 128;
                            }
                        }
                    }
                    /*Je-li posledni, pak typ segmentu bude oznacen inverzne*/
                    if (currentSegment == ae.allSegments.size() - 1) {
                        mark += 128;
                    }

                    header[2] = mark;
                    ls = sct.start;
                    ll = sct.data.length;

                    header[11] = realCounter;
                    realCounter++;

                    header[22] = ls % 256;
                    header[23] = ls / 256;
                    header[24] = ll % 256;
                    header[25] = ll / 256;
                    header[26] = lr % 256;
                    header[27] = lr / 256;
                    p = 183;
                    for (int i = 1; i < 28; i++) {
                        p = p ^ header[i];
                    }
                    header[28] = p;
                    is.add(header);
                    break;
                }
                /*INIT segment*/
                case (Segment.INIT_SEGMENT): {
                    int mark = 73;
                    /*Je-li posledni, pak se typ segmentu bude oznacen inverzne*/
                    if (currentSegment == ae.allSegments.size() - 1) {
                        mark += 128;
                    }

                    header[2] = mark;
                    ls = 738;
                    ll = 2;
                    lr = sct.jump;
                    header[11] = realCounter;
                    realCounter++;
                    header[22] = ls % 256;
                    header[23] = ls / 256;
                    header[24] = ll % 256;
                    header[25] = ll / 256;
                    header[26] = lr % 256;
                    header[27] = lr / 256;
                    p = 183;
                    for (int i = 1; i < 28; i++) {
                        p = p ^ header[i];
                    }
                    header[28] = p;
                    
                    is.add(header);
                    
                    initTag = true;
                    break;
                }
                /*RUN segment*/
                case (Segment.RUN_SEGMENT): {

                    int mark = 82;
                    /*Je-li posledni, pak se typ segmentu bude oznacen inverzne*/
                    if (currentSegment == ae.allSegments.size() - 1) {
                        mark += 128;
                    }

                    header[2] = mark;
                    ls = 736;
                    ll = 2;
                    lr = sct.jump;
                    header[11] = realCounter;
                    realCounter++;
                    header[22] = ls % 256;
                    header[23] = ls / 256;
                    header[24] = ll % 256;
                    header[25] = ll / 256;
                    header[26] = lr % 256;
                    header[27] = lr / 256;
                    p = 183;
                    for (int i = 1; i < 28; i++) {
                        p = p ^ header[i];
                    }
                    header[28] = p;
                    
                    is.add(header);
                    
                    break;
                }
                case (Segment.RUNINIT_SEGMENT): {
                    int mark = 75;
                    /*Je-li posledni, pak se typ segmentu bude oznacen inverzne*/
                    if (currentSegment == ae.allSegments.size() - 1) {
                        mark += 128;
                    }

                    header[2] = mark;
                    ls = 736;
                    ll = 4;
                    lr = 736;

                    header[11] = realCounter;
                    realCounter++;
                    header[22] = ls % 256;
                    header[23] = ls / 256;
                    header[24] = ll % 256;
                    header[25] = ll / 256;
                    header[26] = lr % 256;
                    header[27] = lr / 256;
                    p = 183;
                    for (int i = 1; i < 28; i++) {
                        p = p ^ header[i];
                    }
                    header[28] = p;
                    is.add(header);
                    
                    break;
                }
            }

            /*RealCounter nesmi prekrocit 255*/
            if (realCounter == 255) {
                throw new DOS2BinaryException("Fatal: Chainloader can't handle more than 254 segments");
            }

            /*Hlavicka je zapsana*/
            
            /*Safety pulse + safety pause*/
            is.add(SignalGenerator.INSTR_NARROW);
            is.add(SignalGenerator.INSTR_BLOCKSEP);
            
            
            /*Po hlavicce*/
            is.add(SignalGenerator.INSTR_PILOTTONE);
            is.add(currentPilotLenB);
            is.add(SignalGenerator.INSTR_NARROW);
            is.add(SignalGenerator.INSTR_DATA);

            /*Opet podle segmentu*/
            switch (sct.type) {

                /*Datovy segment*/
                case (Segment.DATA_SEGMENT): {
                    p = 237;

                    is.add(ll + 2);

                    is.add(237);
                    for (int k = 0; k < ll; k++) {
                        p = p ^ sct.data[k];
                    }
                    is.add(sct.data);
                    /*Kontroni soucet*/
                    is.add(p);
                    break;
                }

                /*RUNINIT segment*/
                case (Segment.RUNINIT_SEGMENT): {
                    p = 237;
                    is.add(6);
                    is.add(p);
                    is.add(sct.jump % 256);
                    p = p ^ (sct.jump % 256);
                    is.add(sct.jump / 256);
                    p = p ^ (sct.jump / 256);
                    is.add(sct.jump2 % 256);
                    p = p ^ (sct.jump2 % 256);
                    is.add(sct.jump2 / 256);
                    p = p ^ (sct.jump2 / 256);
                    is.add(p);
                    break;
                }

                /*INIT segment*/
                case (Segment.INIT_SEGMENT): {
                    p = 237;
                    is.add(4);
                    is.add(p);
                    is.add(sct.jump % 256);
                    p = p ^ (sct.jump % 256);
                    is.add(sct.jump / 256);
                    p = p ^ (sct.jump / 256);
                    is.add(p);
                    break;
                }

                case (Segment.RUN_SEGMENT): {
                    p = 237;
                    is.add(4);
                    is.add(p);
                    is.add(sct.jump % 256);
                    p = p ^ (sct.jump % 256);
                    is.add(sct.jump / 256);
                    p = p ^ (sct.jump / 256);
                    is.add(p);
                    break;
                }
            }

            /*Safety pulse + safety pause*/
            is.add(SignalGenerator.INSTR_NARROW);
            is.add(SignalGenerator.INSTR_BLOCKSEP);
            

            /*Mozna prestavka za INIT*/
            if (initTag == true) {
                if (currentPause < plen) {
                    is.add(SignalGenerator.INSTR_LONGSILENCE);
                    is.add(cpi.silenceList[currentPause]);
                    currentPause++;
                }
            }

            
            currentSegment += progress;


        } /*Konec while*/
    }

    private void convertBlockLoading(InstructionStream is, CsTurboPlaylistItem cpi) throws Exception {


        int currentPause = 0;
        int plen = cpi.silenceList.length;


        /*Nejprve se vytvori instance DOS2Binary*/
        DOS2Binary ae = new DOS2Binary(cpi.inFile);
        ae.analyzeFromFile();
        ae.prepareForDumbLoader();


        /*Loader*/
        int plenA = cfgPilotLenA.getAsInt();
        int plenB = cfgPilotLenB.getAsInt();
        int plenK = cfgPilotLenBlockLoading.getAsInt();

        /*Prolongation*/
        if (cfgProlongatePilotTone.getAsBoolean() == true) {
            plenA = PulseCreator.prolongatePilotTone(plenA, 32,cpi.pulses.pilotTonePulseWidth);
            plenB = PulseCreator.prolongatePilotTone(plenB, 32,cpi.pulses.pilotTonePulseWidth);
        }

        if (cfgProlongateBlockLoadingPilotTone.getAsBoolean() == true) {
            plenK = PulseCreator.prolongatePilotTone(plenK,32, cpi.pulses.pilotTonePulseWidth);
        }

        /*Tvorba instrukci pro loader*/
        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(plenA);
        is.add(SignalGenerator.INSTR_SYNC);
        is.add(SignalGenerator.INSTR_DATA);
        is.add(29);


        /*Adresy*/
        int ls = 0;
        int ll = 0;
        int lr = 0;
        lr = superBlock_run;/*8972;*/
        ll = superBlock_data.length;
        ls = superBlock_load;

        int[] header = Utils.createSTHeader(cpi.fname, 3, ls, ll, lr);


        /*Zapsat do souboru*/
        is.add(header);
        

        /*Safety pulse + safety pause*/
        is.add(SignalGenerator.INSTR_NARROW);
        is.add(SignalGenerator.INSTR_BLOCKSEP);

        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(plenB);
        is.add(SignalGenerator.INSTR_SYNC);
        is.add(SignalGenerator.INSTR_DATA);
        is.add(ll + 2);

        /*Data zavadece*/
        int p = 237;
        is.add(237);

        int[] sb_data = new int[superBlock_data.length];
        for(int i=0;i<sb_data.length;i++) {
            sb_data[i]=superBlock_data[i];
        }

        /*Tvorba tabulky, ne vic nez 62 segmentu*/
        int sl = ae.allSegments.size();
        if (sl > 62) {
            throw new DOS2BinaryException("SuperBlock can't handle more than 62 segments");
        }
        Segment s;
        int index = superBlock_table_offset;
        sb_data[superBlock_counter_offset] = sl * 4;
        for (int i = 0; i < sl; i++) {
            s = (Segment) ae.allSegments.get(i);
            sb_data[index] = (s.start) % 256;
            sb_data[index + 1] = (s.start) / 256;
            sb_data[index + 2] = (s.stop + 1) % 256;
            sb_data[index + 3] = (s.stop + 1) / 256;
            index += 4;
        }
        for (int k = 0; k < ll; k++) {
            p = p ^ sb_data[k];
        }
        is.add(sb_data);
        /*Kontroni soucet*/
        is.add(p);
        is.add(SignalGenerator.INSTR_NARROW);

        /*Pauza 3 sekundy*/
        is.add(SignalGenerator.INSTR_LONGSILENCE);
        is.add(3);

        int currentSegment = 0;

        while (currentSegment < ae.allSegments.size()) {

            /*Vezmeme segment*/
            Segment sct = (Segment) ae.allSegments.get(currentSegment);

            is.add(SignalGenerator.INSTR_PILOTTONE);
            is.add(plenK);
            is.add(SignalGenerator.INSTR_SYNC);
            is.add(SignalGenerator.INSTR_DATA);

            p = 183;

            /*Mnozstvi dat: delka segmentu + 2 bajty, zacina se 255*/
            is.add(sct.stop - sct.start + 1 + 2);
            is.add(183);
            for (int k = 0; k < sct.data.length; k++) {
                is.add(sct.data[k]);
                p = p ^ sct.data[k];
            }

            /*Kontroni soucet*/
            is.add(p);

            /*Safety pulse + safety pause*/
            is.add(SignalGenerator.INSTR_NARROW);
            is.add(SignalGenerator.INSTR_BLOCKSEP);

            /*Mozna prestavka za INIT*/
            if (sct.type == Segment.INIT_SEGMENT || sct.type == Segment.RUNINIT_SEGMENT) {
                if (currentPause < plen) {
                    is.add(SignalGenerator.INSTR_LONGSILENCE);
                    is.add(cpi.silenceList[currentPause]);
                    currentPause++;
                }
            }

            
            

            currentSegment++;
        } /*Konec while*/



    }

    private int[] readFileData(String file, int skipBytes, int maxLen) throws Exception {

        byte[] b;
        RandomAccessFile raf = new RandomAccessFile(file, "r");

        int flen = (int) raf.length();
        if (flen > maxLen) {
            raf.close();
            throw new FileTooLargeException("File exceeds " + Integer.toString(maxLen) + " bytes");
        }

        b = new byte[flen - skipBytes];
        raf.skipBytes(skipBytes);
        raf.readFully(b);
        raf.close();


        /*Konverze na int*/
        int[] fileData = new int[b.length];
        for (int i = 0; i < b.length; i++) {
            fileData[i] = b[i] < 0 ? b[i] + 256 : b[i];
        }

        return fileData;
    }

    public String[] getSupportedTapeImageChunks() {
        return new String[] {"trst"};
    }


    /**************************************************************************/
    /**************************************************************************/
    static int superChain_load = 8192;
    static int superChain_run = 8835;
    static int[] superChain_data = {0x20,0xed,0x08,0xa9,0x34,0x8d,0x02,0xd3,0x8d,0x03,0xd3,0x20,0x3f,0x08,0x20,0x50,0x08,0x20,0x36,0x08,0xa5,0x38,0xc9,0xb4,0x90,0xf1,0xa9,0xdd,0x85,0x34,0xa9,0xc2,0x85,
0x32,0xa9,0x08,0x85,0x33,0xa9,0x08,0x85,0x35,0xa9,0xb7,0x20,0xd6,0x07,0x90,0xda,0x18,0xad,0xcc,0x08,0xcd,0xc1,0x08,0xf0,0x10,0x20,0x36,0x08,0xa9,0xff,0x8d,0xfc,
0x02,0xa9,0x01,0x20,0xfc,0xfd,0x4c,0x0b,0x07,0xad,0xd7,0x08,0x85,0x32,0x18,0x6d,0xd9,0x08,0x85,0x34,0xad,0xd8,0x08,0x85,0x33,0x6d,0xda,0x08,0x85,0x35,0xa9,0xed,
0x20,0xd6,0x07,0xb0,0x10,0x20,0x36,0x08,0xa9,0xff,0x8d,0xfc,0x02,0xa9,0x04,0x20,0xfc,0xfd,0x4c,0x0b,0x07,0xa9,0x3c,0x8d,0x02,0xd3,0x8d,0x03,0xd3,0xad,0xc3,0x08,
0x29,0x7f,0xc9,0x52,0xf0,0x37,0xc9,0x44,0xf0,0x33,0xc9,0x49,0xf0,0x26,0xc9,0x58,0xf0,0x07,0xc9,0x59,0xf0,0x12,0x4c,0xbd,0x07,0xad,0xdb,0x08,0x8d,0xe0,0x02,0xad,
0xdc,0x08,0x8d,0xe1,0x02,0x4c,0xc6,0x07,0xad,0xdb,0x08,0x8d,0xe2,0x02,0xad,0xdc,0x08,0x8d,0xe3,0x02,0x20,0xc3,0x07,0x4c,0xc6,0x07,0x6c,0xe2,0x02,0xad,0xc3,0x08,
0x29,0x80,0xf0,0x03,0x6c,0xe0,0x02,0xee,0xc1,0x08,0x4c,0x0b,0x07,0x85,0x36,0x20,0x3f,0x08,0x08,0x20,0x9d,0x08,0xa4,0x39,0x20,0xa2,0x08,0x90,0xf6,0xc4,0x3a,0xb0,
0xf5,0x20,0xa2,0x08,0x90,0x46,0xa4,0x38,0x4c,0x10,0x08,0x28,0xd0,0x08,0xa5,0x36,0x45,0x3c,0xd0,0x39,0xf0,0x0c,0xa0,0x00,0xa5,0x3c,0x91,0x32,0xe6,0x32,0xd0,0x02,
0xe6,0x33,0xa4,0x38,0xc8,0xc8,0x08,0xa9,0x01,0x85,0x3c,0x20,0x9d,0x08,0x90,0x1c,0xc4,0x39,0x26,0x3c,0xa4,0x38,0x90,0xf3,0xa5,0x31,0x45,0x3c,0x85,0x31,0xa5,0x32,
0xc5,0x34,0xa5,0x33,0xe5,0x35,0x90,0xc3,0xa9,0x00,0xc5,0x31,0x68,0xa9,0x40,0x8d,0x0e,0xd4,0x8d,0x0e,0xd2,0x60,0xa0,0x00,0x84,0x30,0x84,0x31,0x8c,0x0e,0xd4,0x8c,
0x00,0xd4,0x8c,0x0e,0xd2,0x18,0x60,0x20,0x9d,0x08,0x90,0xfb,0xa9,0x03,0x85,0x37,0xa0,0x00,0x84,0x40,0xa2,0x03,0x20,0x9d,0x08,0x90,0xec,0xca,0xd0,0xf8,0x98,0x4a,
0x48,0x49,0xff,0x85,0x38,0x68,0x4a,0x48,0x49,0xff,0x85,0x39,0x68,0x4a,0x49,0xff,0x85,0x3a,0xa0,0x00,0x20,0xa2,0x08,0x90,0xce,0xa4,0x38,0x20,0x9d,0x08,0x90,0xc7,
0xc4,0x39,0x90,0xc3,0xe6,0x40,0xd0,0xf1,0xc6,0x37,0x10,0xc4,0xa0,0x00,0x20,0xa2,0x08,0x90,0xb4,0x60,0x20,0xa2,0x08,0x90,0x1d,0xad,0x0a,0xd2,0x29,0xe4,0x05,0x30,
0x4a,0x25,0x37,0x8d,0x1a,0xd0,0xc8,0xf0,0x0d,0xad,0x0f,0xd2,0x29,0x10,0xc5,0x30,0xf0,0xf4,0x85,0x30,0x38,0x60,0x18,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0x53,0x20,0x53,0x55,0x50,0x45,0x52,0x43,0x48,
0x41,0x49,0x4e,0x9b,0xa2,0xdf,0xa0,0x08,0x20,0x42,0xc6,0xa9,0x01,0x8d,0xfc,0x02,0x20,0xfc,0xfd,0xa9,0x00,0x8d,0x44,0x02,0xa9,0x01,0x85,0x09,0x60,0x48,0x08,0x78,
0xad,0x0e,0xd4,0x48,0xa9,0x00,0x8d,0x0e,0xd4,0xad,0x01,0xd3,0x48,0x29,0xfe,0x8d,0x01,0xd3,0xa5,0x80,0x8d,0xfa,0xff,0xa5,0x81,0x8d,0xfb,0xff,0xa5,0x82,0x8d,0xfe,
0xff,0xa5,0x83,0x8d,0xff,0xff,0xa9,0x00,0x85,0x80,0xa9,0x20,0x85,0x81,0xa9,0xfd,0x8d,0xfc,0xff,0xa9,0x01,0x8d,0xfd,0xff,0xa9,0x08,0x85,0x82,0xa9,0x07,0x85,0x83,
0xa0,0x00,0xb1,0x80,0x91,0x82,0xce,0xfc,0xff,0xae,0xfc,0xff,0xe0,0xff,0xd0,0x0a,0xce,0xfd,0xff,0xae,0xfd,0xff,0xe0,0xff,0xf0,0x09,0xc8,0xd0,0xe5,0xe6,0x81,0xe6,
0x83,0xd0,0xdf,0x68,0x8d,0x01,0xd3,0x68,0x8d,0x0e,0xd4,0xad,0xfa,0xff,0x85,0x80,0xad,0xfb,0xff,0x85,0x81,0xad,0xfe,0xff,0x85,0x82,0xad,0xff,0xff,0x85,0x83,0x28,
0x68,0x60,0x20,0xfe,0x21,0x4c,0x08,0x07,0x00};

    static int superBlock_load = 8192;
    static int superBlock_run = 8992;
    static int superBlock_table_offset = 0x17C;
    static int superBlock_counter_offset = 0x17B;
    static int[] superBlock_data = {0x20,0x8a,0x0a,0xa2,0x0c,0x8e,0xfc,0x02,0xa9,0x01,0x20,0xfc,0xfd,0xa9,0x00,0x8d,0x78,0x09,0xad,0x78,0x09,0xcd,0x7b,0x09,0xd0,0x03,0x6c,0xe0,0x02,0xa9,0x79,0x8d,0xe2,
0x02,0xa9,0x09,0x8d,0xe3,0x02,0xac,0x78,0x09,0xb9,0x7c,0x09,0x85,0x32,0xc8,0xb9,0x7c,0x09,0x85,0x33,0xc8,0xb9,0x7c,0x09,0x85,0x34,0xc8,0xb9,0x7c,0x09,0x85,0x35,
0xa9,0x34,0x8d,0x02,0xd3,0x8d,0x03,0xd3,0x20,0x78,0x08,0xa9,0x3c,0x8d,0x02,0xd3,0x8d,0x03,0xd3,0xb0,0x0d,0xa9,0xff,0x8d,0xfc,0x02,0xa9,0x04,0x20,0xfc,0xfd,0x4c,
0x0d,0x08,0x20,0x75,0x08,0xee,0x78,0x09,0xee,0x78,0x09,0xee,0x78,0x09,0xee,0x78,0x09,0x4c,0x12,0x08,0x6c,0xe2,0x02,0x20,0xf6,0x08,0x20,0x07,0x09,0x20,0xed,0x08,
0xa5,0x38,0xc9,0xb4,0x90,0xf1,0xa9,0xb7,0x20,0x8d,0x08,0x60,0x85,0x36,0x20,0xf6,0x08,0x08,0x20,0x54,0x09,0xa4,0x39,0x20,0x59,0x09,0x90,0xf6,0xc4,0x3a,0xb0,0xf5,
0x20,0x59,0x09,0x90,0x46,0xa4,0x38,0x4c,0xc7,0x08,0x28,0xd0,0x08,0xa5,0x36,0x45,0x3c,0xd0,0x39,0xf0,0x0c,0xa0,0x00,0xa5,0x3c,0x91,0x32,0xe6,0x32,0xd0,0x02,0xe6,
0x33,0xa4,0x38,0xc8,0xc8,0x08,0xa9,0x01,0x85,0x3c,0x20,0x54,0x09,0x90,0x1c,0xc4,0x39,0x26,0x3c,0xa4,0x38,0x90,0xf3,0xa5,0x31,0x45,0x3c,0x85,0x31,0xa5,0x32,0xc5,
0x34,0xa5,0x33,0xe5,0x35,0x90,0xc3,0xa9,0x00,0xc5,0x31,0x68,0xa9,0x40,0x8d,0x0e,0xd4,0x8d,0x0e,0xd2,0x60,0xa0,0x00,0x84,0x30,0x84,0x31,0x8c,0x0e,0xd4,0x8c,0x00,
0xd4,0x8c,0x0e,0xd2,0x18,0x60,0x20,0x54,0x09,0x90,0xfb,0xa9,0x03,0x85,0x37,0xa0,0x00,0x84,0x40,0xa2,0x03,0x20,0x54,0x09,0x90,0xec,0xca,0xd0,0xf8,0x98,0x4a,0x48,
0x49,0xff,0x85,0x38,0x68,0x4a,0x48,0x49,0xff,0x85,0x39,0x68,0x4a,0x49,0xff,0x85,0x3a,0xa0,0x00,0x20,0x59,0x09,0x90,0xce,0xa4,0x38,0x20,0x54,0x09,0x90,0xc7,0xc4,
0x39,0x90,0xc3,0xe6,0x40,0xd0,0xf1,0xc6,0x37,0x10,0xc4,0xa0,0x00,0x20,0x59,0x09,0x90,0xb4,0x60,0x20,0x59,0x09,0x90,0x1d,0xad,0x0a,0xd2,0x29,0xe4,0x05,0x30,0x4a,
0x25,0x37,0x8d,0x1a,0xd0,0xc8,0xf0,0x0d,0xad,0x0f,0xd2,0x29,0x10,0xc5,0x30,0xf0,0xf4,0x85,0x30,0x38,0x60,0x18,0x60,0x00,0x60,0xff,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x54,0x53,0x20,0x53,0x55,
0x50,0x45,0x52,0x42,0x4c,0x4f,0x43,0x4b,0x9b,0xa2,0x7c,0xa0,0x0a,0x20,0x42,0xc6,0xa9,0x00,0x8d,0x44,0x02,0xa9,0x01,0x85,0x09,0x60,0x48,0x08,0x78,0xad,0x0e,0xd4,
0x48,0xa9,0x00,0x8d,0x0e,0xd4,0xad,0x01,0xd3,0x48,0x29,0xfe,0x8d,0x01,0xd3,0xa5,0x80,0x8d,0xfa,0xff,0xa5,0x81,0x8d,0xfb,0xff,0xa5,0x82,0x8d,0xfe,0xff,0xa5,0x83,
0x8d,0xff,0xff,0xa9,0x00,0x85,0x80,0xa9,0x20,0x85,0x81,0xa9,0x9a,0x8d,0xfc,0xff,0xa9,0x02,0x8d,0xfd,0xff,0xa9,0x00,0x85,0x82,0xa9,0x08,0x85,0x83,0xa0,0x00,0xb1,
0x80,0x91,0x82,0xce,0xfc,0xff,0xae,0xfc,0xff,0xe0,0xff,0xd0,0x0a,0xce,0xfd,0xff,0xae,0xfd,0xff,0xe0,0xff,0xf0,0x09,0xc8,0xd0,0xe5,0xe6,0x81,0xe6,0x83,0xd0,0xdf,
0x68,0x8d,0x01,0xd3,0x68,0x8d,0x0e,0xd4,0xad,0xfa,0xff,0x85,0x80,0xad,0xfb,0xff,0x85,0x81,0xad,0xfe,0xff,0x85,0x82,0xad,0xff,0xff,0x85,0x83,0x28,0x68,0x60,0x20,
0x9b,0x22,0x4c,0x00,0x08,0x00};

    public void addInstructionsForChunk(InstructionStream is, TapeImageChunk chunk) throws Exception {

        /*trXX chunk*/
        if (chunk instanceof TurboChunk) {
            int bdr = ((TurboChunk)(chunk)).getBaudRate();

            /*Upkeep baud rate range*/
            if (bdr<2270) bdr=2270;
            if (bdr>9600) bdr=9600;

            /*Choose closest known pulses*/
            PulseCrate speed = Utils.getNearestBaudRate(bdr, Turgen.pulseCache.getPulsesForPlugin(this));

            /*Generate setup instruction*/
            Utils.addSetupInstruction(is, speed, chunk.getType(),SignalGenerator.FLAG_POLARITY_10,SignalGenerator.FLAG_ORDER_HL);

            return;
        }

        /*Only data chunk accepted*/
        if (!(chunk instanceof DataChunk)) return;
        DataChunk dataChunk = (DataChunk)chunk;
        


        /*Zavadeci ton a data bloku*/
        is.add(SignalGenerator.INSTR_PILOTTONE);
        is.add(dataChunk.getAux());

        /*Kratky pulz*/
        is.add(SignalGenerator.INSTR_NARROW);

        /*Data bloku*/
        is.add(SignalGenerator.INSTR_DATA);
        is.add(dataChunk.getLength());

        int[] dta = dataChunk.getData();

        for(int i=0;i<dataChunk.getLength();i++) {
            is.add(dta[i]);
        }

        /*Bezpecnostni pulz*/
        is.add(SignalGenerator.INSTR_NARROW);
        
    }

     public BinaryConvertor[] getBinaryConvertors() {
        BinaryConvertor[] bc = new BinaryConvertor[3];
        MonolithicBinaryConvertor mbc = new MonolithicBinaryConvertor(true,this);
        ChainloadingBinaryConvertor cbc = new ChainloadingBinaryConvertor(true, 1800, 2270,this);
        BlockloadingBinaryConvertor bbc = new BlockloadingBinaryConvertor(true, 2048, 2682,this);
        bc[0]=mbc;
        bc[1]=cbc;
        bc[2]=bbc;
        return bc;
    }

     public String getCountry() {
        return "CS";
    }

    public boolean isUsingAlternateFileChooser() {
        return false;
    }

    
}
